@@ -54,6 +54,7 @@ ArrayDirectory::ArrayDirectory(
     const URI& uri,
     uint64_t timestamp_start,
     uint64_t timestamp_end,
+    bool consolidation_with_timestamps_config,
     ArrayDirectoryMode mode)
     : uri_(uri.add_trailing_slash())
     , vfs_(vfs)
@@ -61,7 +62,9 @@ ArrayDirectory::ArrayDirectory(
     , timestamp_start_(timestamp_start)
     , timestamp_end_(timestamp_end)
     , mode_(mode)
-    , loaded_(false) {
+    , loaded_(false)
+    , consolidation_with_timestamps_config_(
+          consolidation_with_timestamps_config) {
   auto st = load();
   if (!st.ok()) {
     throw std::logic_error(st.message());
@@ -668,6 +671,29 @@ std::vector<URI> ArrayDirectory::compute_fragment_meta_uris(
   return ret;
 }
 
+bool ArrayDirectory::timestamps_overlap(
+    const std::pair<uint64_t, uint64_t> fragment_timestamp_range,
+    const bool consolidation_with_timestamps) const {
+  auto fragment_timestamp_start = fragment_timestamp_range.first;
+  auto fragment_timestamp_end = fragment_timestamp_range.second;
+
+  if (!consolidation_with_timestamps) {
+    // True if the fragment falls fully within start and end times
+    auto full_overlap = fragment_timestamp_start >= timestamp_start_ &&
+                        fragment_timestamp_end <= timestamp_end_;
+    return full_overlap;
+  } else {
+    // When consolidated fragment has timestamps, true if there is even partial
+    // overlap
+    auto partial_overlap =
+        ((fragment_timestamp_start >= timestamp_start_ &&
+          fragment_timestamp_start <= timestamp_end_) ||
+         (fragment_timestamp_end >= timestamp_start_ &&
+          fragment_timestamp_end <= timestamp_end_));
+    return partial_overlap;
+  }
+}
+
 tuple<Status, optional<std::vector<URI>>, optional<std::vector<URI>>>
 ArrayDirectory::compute_uris_to_vacuum(const std::vector<URI>& uris) const {
   // Get vacuum URIs
@@ -675,19 +701,22 @@ ArrayDirectory::compute_uris_to_vacuum(const std::vector<URI>& uris) const {
   std::unordered_set<std::string> non_vac_uris_set;
   std::unordered_map<std::string, size_t> uris_map;
   for (size_t i = 0; i < uris.size(); ++i) {
-    std::pair<uint64_t, uint64_t> timestamp_range;
+    // Get the start and end timestamp for this fragment
+    std::pair<uint64_t, uint64_t> fragment_timestamp_range;
     RETURN_NOT_OK_TUPLE(
-        utils::parse::get_timestamp_range(uris[i], &timestamp_range),
+        utils::parse::get_timestamp_range(uris[i], &fragment_timestamp_range),
         nullopt,
         nullopt);
-
     if (is_vacuum_file(uris[i])) {
-      if (timestamp_range.first >= timestamp_start_ &&
-          timestamp_range.second <= timestamp_end_)
+      if (timestamps_overlap(
+              fragment_timestamp_range,
+              consolidation_with_timestamps_supported(uris[i]))) {
         vac_files.emplace_back(uris[i]);
+      }
     } else {
-      if (timestamp_range.first < timestamp_start_ ||
-          timestamp_range.second > timestamp_end_) {
+      if (!timestamps_overlap(
+              fragment_timestamp_range,
+              consolidation_with_timestamps_supported(uris[i]))) {
         non_vac_uris_set.emplace(uris[i].to_string());
       } else {
         uris_map[uris[i].to_string()] = i;
@@ -765,16 +794,16 @@ ArrayDirectory::compute_filtered_uris(
     if (is_vacuum_file(uri))
       continue;
 
-    // Add only URIs whose first timestamp is greater than or equal to the
-    // timestamp_start and whose second timestamp is smaller than or equal to
-    // the timestamp_end
-    std::pair<uint64_t, uint64_t> timestamp_range;
+    // Get the start and end timestamp for this fragment
+    std::pair<uint64_t, uint64_t> fragment_timestamp_range;
     RETURN_NOT_OK_TUPLE(
-        utils::parse::get_timestamp_range(uri, &timestamp_range), nullopt);
-    auto t1 = timestamp_range.first;
-    auto t2 = timestamp_range.second;
-    if (t1 >= timestamp_start_ && t2 <= timestamp_end_)
-      filtered_uris.emplace_back(uri, timestamp_range);
+        utils::parse::get_timestamp_range(uri, &fragment_timestamp_range),
+        nullopt);
+    if (timestamps_overlap(
+            fragment_timestamp_range,
+            consolidation_with_timestamps_supported(uri))) {
+      filtered_uris.emplace_back(uri, fragment_timestamp_range);
+    }
   }
 
   // Sort the names based on the timestamps
@@ -865,5 +894,19 @@ Status ArrayDirectory::is_fragment(
   return Status::Ok();
 }
 
+bool ArrayDirectory::consolidation_with_timestamps_supported(
+    const URI& uri) const {
+  // Get the fragment version from the uri
+  uint32_t version;
+  auto name = uri.remove_trailing_slash().last_path_part();
+  utils::parse::get_fragment_version(name, &version);
+
+  // get_fragment_version returns UINT32_MAX for versions <= 2 so we should
+  // explicitly exclude this case when checking if consolidation with timestamps
+  // is supported on a fragment
+  return consolidation_with_timestamps_config_ &&
+         mode_ == ArrayDirectoryMode::READ && version >= 14 &&
+         version != UINT32_MAX;
+}
 }  // namespace sm
 }  // namespace tiledb
