@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -114,6 +114,13 @@ Status DenseReader::init() {
   // Check the validity buffer sizes.
   RETURN_NOT_OK(check_validity_buffer_sizes());
 
+  bool found = false;
+  uint64_t tile_cache_size = 0;
+  RETURN_NOT_OK(
+      config_.get<uint64_t>("sm.tile_cache_size", &tile_cache_size, &found));
+  assert(found);
+  disable_cache_ = tile_cache_size == 0;
+
   return Status::Ok();
 }
 
@@ -186,7 +193,7 @@ void DenseReader::reset() {
 
 template <class OffType>
 Status DenseReader::dense_read() {
-  auto type = array_schema_.domain().dimension(0)->type();
+  auto type{array_schema_.domain().dimension_ptr(0)->type()};
   switch (type) {
     case Datatype::INT8:
       return dense_read<int8_t, OffType>();
@@ -368,7 +375,7 @@ Status DenseReader::dense_read() {
   RETURN_CANCEL_OR_ERROR(st);
 
   // Copy attribute data to users buffers.
-  status = read_attributes<DimType, OffType>(
+  status = copy_attributes<DimType, OffType>(
       fixed_names,
       var_names,
       subarray,
@@ -499,8 +506,9 @@ DenseReader::apply_query_condition(
     std::vector<uint64_t>& tile_offsets,
     const std::vector<RangeInfo>& range_info,
     std::map<const DimType*, ResultSpaceTile<DimType>>& result_space_tiles) {
+  auto timer_se = stats_->start_timer("apply_query_condition");
   std::vector<uint8_t> qc_result;
-  if (!condition_.clauses().empty()) {
+  if (!condition_.empty()) {
     // For easy reference.
     const auto& tile_coords = subarray.tile_coords();
     const auto cell_num = subarray.cell_num();
@@ -555,7 +563,9 @@ DenseReader::apply_query_condition(
                 it->second,
                 cell_slab.coords_.data());
 
-            for (int32_t i = (int32_t)frag_domains.size() - 1; i >= 0; --i) {
+            for (int32_t i = static_cast<int32_t>(frag_domains.size()) - 1;
+                 i >= 0;
+                 --i) {
               // If the cell slab overlaps this fragment domain range, apply
               // clause.
               auto&& [overlaps, start, end] = cell_slab_overlaps_range(
@@ -564,6 +574,13 @@ DenseReader::apply_query_condition(
                   cell_slab.coords_.data(),
                   cell_slab.length_);
               if (overlaps) {
+                // Re-initialize the bitmap to 1 in case of overlapping domains.
+                if (i != static_cast<int32_t>(frag_domains.size()) - 1) {
+                  for (uint64_t c = start; c <= end; c++) {
+                    dest_ptr[c] = 1;
+                  }
+                }
+
                 RETURN_NOT_OK(condition_.apply_dense(
                     *(fragment_metadata_[frag_domains[i].first]
                           ->array_schema()
@@ -631,7 +648,7 @@ uint64_t DenseReader::fix_offsets_buffer(
 }
 
 template <class DimType, class OffType>
-Status DenseReader::read_attributes(
+Status DenseReader::copy_attributes(
     const std::vector<std::string>& fixed_names,
     const std::vector<std::string>& var_names,
     const Subarray& subarray,
@@ -640,6 +657,8 @@ Status DenseReader::read_attributes(
     const std::vector<RangeInfo>& range_info,
     std::map<const DimType*, ResultSpaceTile<DimType>>& result_space_tiles,
     const std::vector<uint8_t>& qc_result) {
+  auto timer_se = stats_->start_timer("copy_attributes");
+
   // For easy reference
   const auto& tile_coords = subarray.tile_coords();
   const auto cell_num = subarray.cell_num();
@@ -683,79 +702,94 @@ Status DenseReader::read_attributes(
     }
 
     // Process offsets in parallel.
-    auto status = parallel_for(
-        storage_manager_->compute_tp(), 0, tile_coords.size(), [&](uint64_t t) {
-          // Find out result space tile and tile subarray.
-          const DimType* tc = (DimType*)&tile_coords[t][0];
-          auto it = result_space_tiles.find(tc);
-          assert(it != result_space_tiles.end());
+    {
+      auto timer_se = stats_->start_timer("copy_offset_tiles");
+      auto status = parallel_for(
+          storage_manager_->compute_tp(),
+          0,
+          tile_coords.size(),
+          [&](uint64_t t) {
+            // Find out result space tile and tile subarray.
+            const DimType* tc = (DimType*)&tile_coords[t][0];
+            auto it = result_space_tiles.find(tc);
+            assert(it != result_space_tiles.end());
 
-          // Copy the tile offsets.
-          return copy_offset_tiles<DimType, OffType>(
-              var_names,
-              dst_off_bufs,
-              dst_val_bufs,
-              attributes,
-              data_type_sizes,
-              it->second,
-              subarray,
-              tile_subarrays[t],
-              global_order ? tile_offsets[t] : 0,
-              var_data,
-              range_info,
-              qc_result);
-        });
-    RETURN_NOT_OK(status);
+            // Copy the tile offsets.
+            return copy_offset_tiles<DimType, OffType>(
+                var_names,
+                dst_off_bufs,
+                dst_val_bufs,
+                attributes,
+                data_type_sizes,
+                it->second,
+                subarray,
+                tile_subarrays[t],
+                global_order ? tile_offsets[t] : 0,
+                var_data,
+                range_info,
+                qc_result);
+          });
+      RETURN_NOT_OK(status);
+    }
 
     // We have the cell lengths in the users buffer, convert to offsets.
     std::vector<uint64_t> var_buffer_sizes(var_names.size());
-    status = parallel_for(
-        storage_manager_->compute_tp(), 0, var_names.size(), [&](uint64_t n) {
-          const auto& name = var_names[n];
-          const bool nullable = array_schema_.is_nullable(name);
-          var_buffer_sizes[n] = fix_offsets_buffer<OffType>(
-              name, nullable, cell_num, var_data[n]);
+    {
+      auto timer_se = stats_->start_timer("fix_offset_tiles");
+      auto status = parallel_for(
+          storage_manager_->compute_tp(), 0, var_names.size(), [&](uint64_t n) {
+            const auto& name = var_names[n];
+            const bool nullable = array_schema_.is_nullable(name);
+            var_buffer_sizes[n] = fix_offsets_buffer<OffType>(
+                name, nullable, cell_num, var_data[n]);
 
-          // Make sure the user var buffer is big enough.
-          uint64_t required_var_size = var_buffer_sizes[n];
-          if (elements_mode_)
-            required_var_size *= datatype_size(array_schema_.type(name));
+            // Make sure the user var buffer is big enough.
+            uint64_t required_var_size = var_buffer_sizes[n];
+            if (elements_mode_)
+              required_var_size *= datatype_size(array_schema_.type(name));
 
-          // Exit early in case of overflow.
-          if (read_state_.overflowed_ ||
-              required_var_size > *buffers_[name].buffer_var_size_) {
-            read_state_.overflowed_ = true;
+            // Exit early in case of overflow.
+            if (read_state_.overflowed_ ||
+                required_var_size > *buffers_[name].buffer_var_size_) {
+              read_state_.overflowed_ = true;
+              return Status::Ok();
+            }
+
+            *buffers_[name].buffer_var_size_ = required_var_size;
             return Status::Ok();
-          }
-
-          *buffers_[name].buffer_var_size_ = required_var_size;
-          return Status::Ok();
-        });
-    RETURN_NOT_OK(status);
+          });
+      RETURN_NOT_OK(status);
+    }
 
     if (read_state_.overflowed_) {
       return Status::Ok();
     }
 
-    // Process var data in parallel.
-    status = parallel_for(
-        storage_manager_->compute_tp(), 0, tile_coords.size(), [&](uint64_t t) {
-          return copy_var_tiles<DimType, OffType>(
-              var_names,
-              dst_var_bufs,
-              dst_off_bufs,
-              data_type_sizes,
-              subarray,
-              tile_subarrays[t],
-              global_order ? tile_offsets[t] : 0,
-              var_data,
-              range_info,
-              t == tile_coords.size() - 1,
-              var_buffer_sizes);
+    {
+      auto timer_se = stats_->start_timer("copy_var_tiles");
+      // Process var data in parallel.
+      auto status = parallel_for(
+          storage_manager_->compute_tp(),
+          0,
+          tile_coords.size(),
+          [&](uint64_t t) {
+            return copy_var_tiles<DimType, OffType>(
+                var_names,
+                dst_var_bufs,
+                dst_off_bufs,
+                data_type_sizes,
+                subarray,
+                tile_subarrays[t],
+                global_order ? tile_offsets[t] : 0,
+                var_data,
+                range_info,
+                t == tile_coords.size() - 1,
+                var_buffer_sizes);
 
-          return Status::Ok();
-        });
-    RETURN_NOT_OK(status);
+            return Status::Ok();
+          });
+      RETURN_NOT_OK(status);
+    }
   }
 
   if (!fixed_names.empty()) {
@@ -785,31 +819,37 @@ Status DenseReader::read_attributes(
       cell_sizes.push_back(array_schema_.cell_size(name));
     }
 
-    // Process values in parallel.
-    auto status = parallel_for(
-        storage_manager_->compute_tp(), 0, tile_coords.size(), [&](uint64_t t) {
-          // Find out result space tile and tile subarray.
-          const DimType* tc = (DimType*)&tile_coords[t][0];
-          auto it = result_space_tiles.find(tc);
-          assert(it != result_space_tiles.end());
+    {
+      auto timer_se = stats_->start_timer("copy_fixed_tiles");
+      // Process values in parallel.
+      auto status = parallel_for(
+          storage_manager_->compute_tp(),
+          0,
+          tile_coords.size(),
+          [&](uint64_t t) {
+            // Find out result space tile and tile subarray.
+            const DimType* tc = (DimType*)&tile_coords[t][0];
+            auto it = result_space_tiles.find(tc);
+            assert(it != result_space_tiles.end());
 
-          // Copy the tile fixed values.
-          RETURN_NOT_OK(copy_fixed_tiles(
-              fixed_names,
-              dst_bufs,
-              dst_val_bufs,
-              attributes,
-              cell_sizes,
-              it->second,
-              subarray,
-              tile_subarrays[t],
-              global_order ? tile_offsets[t] : 0,
-              range_info,
-              qc_result));
+            // Copy the tile fixed values.
+            RETURN_NOT_OK(copy_fixed_tiles(
+                fixed_names,
+                dst_bufs,
+                dst_val_bufs,
+                attributes,
+                cell_sizes,
+                it->second,
+                subarray,
+                tile_subarrays[t],
+                global_order ? tile_offsets[t] : 0,
+                range_info,
+                qc_result));
 
-          return Status::Ok();
-        });
-    RETURN_NOT_OK(status);
+            return Status::Ok();
+          });
+      RETURN_NOT_OK(status);
+    }
 
     // Set the output size for the fixed buffer.
     for (auto& name : fixed_names) {
@@ -900,7 +940,7 @@ uint64_t DenseReader::get_dest_cell_offset_row_col(
     for (int32_t d = 0; d < dim_num; d++) {
       auto r = converted_range_coords[d];
       auto min = *static_cast<const DimType*>(
-          subarray.ranges_for_dim((uint32_t)d)[r].start());
+          subarray.ranges_for_dim((uint32_t)d)[r].start_fixed());
       ret += range_info[d].multiplier_ *
              (coords[d] - min + range_info[d].cell_offsets_[r]);
     }
@@ -908,7 +948,7 @@ uint64_t DenseReader::get_dest_cell_offset_row_col(
     for (int32_t d = dim_num - 1; d >= 0; d--) {
       auto r = converted_range_coords[d];
       auto min = *static_cast<const DimType*>(
-          subarray.ranges_for_dim((uint32_t)d)[r].start());
+          subarray.ranges_for_dim((uint32_t)d)[r].start_fixed());
       ret += range_info[d].multiplier_ *
              (coords[d] - min + range_info[d].cell_offsets_[r]);
     }
