@@ -73,7 +73,8 @@ class SparseGlobalOrderReader : public SparseIndexReaderBase,
       std::unordered_map<std::string, QueryBuffer>& buffers,
       Subarray& subarray,
       Layout layout,
-      QueryCondition& condition);
+      QueryCondition& condition,
+      bool consolidation_with_timestamps);
 
   /** Destructor. */
   ~SparseGlobalOrderReader() = default;
@@ -155,9 +156,15 @@ class SparseGlobalOrderReader : public SparseIndexReaderBase,
   /** Memory budget per fragment for qc tiles. */
   double per_fragment_qc_memory_;
 
+  /** Enables consolidation with timestamps or not. */
+  bool consolidation_with_timestamps_;
+
   /** Mutex to protect the tile queue. */
   std::mutex tile_queue_mutex_;
 
+  /** Stores last cell for fragments consolidated with timestamps. */
+  std::vector<FragIdx> last_cells_;
+
   /* ********************************* */
   /*       PRIVATE DECLARATIONS        */
   /* ********************************* */
@@ -203,6 +210,25 @@ class SparseGlobalOrderReader : public SparseIndexReaderBase,
    */
   tuple<Status, optional<bool>> create_result_tiles();
 
+  /**
+   * Process tiles with timestamps to deduplicate entries.
+   *
+   * @param result_tiles Result tiles to process.
+   *
+   * @return Status.
+   */
+  Status dedup_tiles_with_timestamps(std::vector<ResultTile*>& result_tiles);
+
+  /**
+   * Process fragments with timestamps to deduplicate entries.
+   * This removes cells across tiles.
+   *
+   * @param result_tiles Result tiles to process.
+   *
+   * @return Status.
+   */
+  Status dedup_fragments_with_timestamps();
+
   /**
    * Populate a result cell slab to process.
    *
@@ -211,10 +237,29 @@ class SparseGlobalOrderReader : public SparseIndexReaderBase,
   tuple<Status, optional<std::vector<ResultCellSlab>>>
   compute_result_cell_slab();
 
+  /**
+   * Is the result coord the last cell of a consolidated fragment with
+   * timestamps.
+   *
+   * @param frag_idx Fragment index for the result coords.
+   * @param rc Result coords.
+   *
+   * @return true if the result coords is the last cell of a consolidated
+   * fragment with timestamps.
+   */
+  inline bool last_in_memory_cell_of_consolidated_fragment(
+      const unsigned int frag_idx, const GlobalOrderResultCoords& rc) const {
+    return !all_tiles_loaded_[frag_idx] &&
+           fragment_metadata_[frag_idx]->has_timestamps() &&
+           rc.tile_->tile_idx() == last_cells_[frag_idx].tile_idx_ &&
+           rc.pos_ == last_cells_[frag_idx].cell_idx_;
+  }
+
   /**
    * Add a cell (for a specific fragment) to the queue of cells currently being
    * processed.
    *
+   * @param dups Are we returning dups or not.
    * @param rc Current result coords for the fragment.
    * @param result_tiles_it Iterator, per frag, in the list of retult tiles.
    * @param tile_queue Queue of one result coords, per fragment, sorted.
@@ -223,6 +268,7 @@ class SparseGlobalOrderReader : public SparseIndexReaderBase,
    */
   template <class CompType>
   tuple<Status, optional<bool>> add_next_cell_to_queue(
+      bool dups,
       GlobalOrderResultCoords& rc,
       std::vector<TileListIt>& result_tiles_it,
       TileMinHeap<CompType>& tile_queue);
@@ -236,6 +282,15 @@ class SparseGlobalOrderReader : public SparseIndexReaderBase,
    */
   Status compute_hilbert_values(std::vector<ResultTile*>& result_tiles);
 
+  /**
+   * Get the timestamp value for a result coords.
+   *
+   * @param rc Result coords.
+   *
+   * @return timestamp.
+   */
+  uint64_t get_timestamp(const GlobalOrderResultCoords& rc) const;
+
   /**
    * Compute the result cell slabs once tiles are loaded.
    *
@@ -340,6 +395,22 @@ class SparseGlobalOrderReader : public SparseIndexReaderBase,
       const std::vector<uint64_t>& cell_offsets,
       QueryBuffer& query_buffer);
 
+  /**
+   * Copy timestamps tiles.
+   *
+   * @param num_range_threads Total number of range threads.
+   * @param result_cell_slabs Result cell slabs to process.
+   * @param cell_offsets Cell offset per result tile.
+   * @param query_buffer Query buffer to operate on.
+   *
+   * @return Status.
+   */
+  Status copy_timestamps_tiles(
+      const uint64_t num_range_threads,
+      const std::vector<ResultCellSlab>& result_cell_slabs,
+      const std::vector<uint64_t>& cell_offsets,
+      QueryBuffer& query_buffer);
+
   /**
    * Make sure we respect memory budget for copy operation by making sure that,
    * for all attributes to be copied, the size of tiles in memory can fit into
